
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: ChatEdit_UpdateHeader, GetChannelName, IsControlKeyDown, IsInGroup, IsInGuild, IsInInstance, IsInRaid, IsShiftKeyDown, LE_PARTY_CATEGORY_HOME, LE_PARTY_CATEGORY_INSTANCE, strsub, tostring

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat Cycle")

local CHAT_CHANNEL_CYCLE = {
	"SAY",
	"GUILD",
	"INSTANCE_CHAT",
	"PARTY",
	"RAID",
	"OFFICER",
	"YELL",
	"SMART_WHISPER",
	"CHANNEL1",
	"CHANNEL2",
	"CHANNEL3",
	"CHANNEL4",
	"CHANNEL5",
	"CHANNEL6",
	"CHANNEL7",
	"CHANNEL8",
	"CHANNEL9",
	"CHANNEL10",
	"CHANNEL11",
	"CHANNEL12",
	"CHANNEL13",
	"CHANNEL14",
	"CHANNEL15",
	"CHANNEL16",
	"CHANNEL17",
	"CHANNEL18",
	"CHANNEL19",
	"CHANNEL20",
}

for index, key in next, CHAT_CHANNEL_CYCLE do
	CHAT_CHANNEL_CYCLE[key] = index
end

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function CanUseInstanceChat()
	return IsInInstance() and (IsInGroup(LE_PARTY_CATEGORY_INSTANCE) or IsInRaid(LE_PARTY_CATEGORY_INSTANCE))
end

local function CanUsePartyChat()
	return IsInGroup() and IsInGroup(LE_PARTY_CATEGORY_HOME)
end

local function CanUseRaidChat()
	return IsInRaid() and IsInRaid(LE_PARTY_CATEGORY_HOME)
end

local function GetChannelIndex(chatType)
	return chatType:match("^CHANNEL(%d+)$")
end

local function CanUseChannel(channelIndex)
	return GetChannelName(channelIndex) ~= 0
end

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	if not self:IsHooked("ChatEdit_CustomTabPressed") then
		self:RawHook("ChatEdit_CustomTabPressed", true)
	end

	if not self:IsHooked("ChatEdit_SecureTabPressed") then
		self:RawHook("ChatEdit_SecureTabPressed", true)
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:ChatEdit_CustomTabPressed(editBox)
	-- Ignore when performing command completion.
	if strsub(tostring(editBox:GetText()), 1, 1) == "/" then
		return
	end

	-- Ignore when holding control.
	if IsControlKeyDown() then
		return
	end

	-- Reverse the cycle if the Shift key is down.
	local increment = IsShiftKeyDown() and -1 or 1

	-- Get the active chat information.
	local channelIndex = editBox:GetAttribute("channelTarget")
	local chatType = editBox:GetAttribute("chatType")
	local tellTarget = editBox:GetAttribute("tellTarget")

	if chatType == "WHISPER" then
		chatType = "SMART_WHISPER"
	elseif chatType == "CHANNEL" then
		chatType = "CHANNEL" .. channelIndex
	elseif chatType == "BN_WHISPER" then
		chatType = "SMART_WHISPER"
	end

	-- Cycle to the new chat type.
	for i = 1, #CHAT_CHANNEL_CYCLE + 1 do
		local index = (CHAT_CHANNEL_CYCLE[chatType] or 0) + increment
		if index < 1 then index = #CHAT_CHANNEL_CYCLE end
		if index > #CHAT_CHANNEL_CYCLE then index = 1 end

		chatType = CHAT_CHANNEL_CYCLE[index]
		channelIndex = GetChannelIndex(chatType)

		-- Skip unavailable chat types.
		if chatType == "GUILD" and not IsInGuild() then
		elseif chatType == "INSTANCE_CHAT" and not CanUseInstanceChat() then
		elseif chatType == "PARTY" and not CanUsePartyChat() then
		elseif chatType == "RAID" and not CanUseRaidChat() then
		elseif chatType == "OFFICER" and not IsInGuild() then
		elseif chatType == "SMART_WHISPER" and not tellTarget then
		elseif channelIndex and not CanUseChannel(channelIndex) then
		-- Set the chat type.
		else
			if channelIndex then
				editBox:SetAttribute("channelTarget", channelIndex)
				editBox:SetAttribute("chatType", "CHANNEL")
				editBox:SetAttribute("stickyType", "CHANNEL")
				ChatEdit_UpdateHeader(editBox)
				break
			else
				editBox:SetAttribute("chatType", chatType)
				ChatEdit_UpdateHeader(editBox)
				break
			end
		end
	end
end

function M:ChatEdit_SecureTabPressed(frame)
	if not IsControlKeyDown() then
		return
	end

	self.hooks.ChatEdit_SecureTabPressed(frame)
end
